`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/05/15 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////
`include "global.vh"

module topmodule(
    D_ssd,
    ssd_ctl,
    clk,
    rst,
    PS2_DATA,
    PS2_CLK
    );

    output [7:0] D_ssd;
    output [3:0] ssd_ctl;
    input clk;
    input rst;
    inout PS2_CLK;
    inout PS2_DATA;
    
    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire [511:0] key_down;
    wire [8:0] last_change;
    wire key_valid;
    wire [3:0] decode_result;
    
    wire valid_signal;

    wire [1:0] instruction;

    wire [3:0] digit_0, digit_1, digit_2, digit_3;
    wire [3:0] add_digit_2, add_digit_1, add_digit_0;
    wire [3:0] subtract_digit_1, subtract_digit_0;
    wire [3:0] multiply_digit_3, multiply_digit_2, multiply_digit_1, multiply_digit_0;

    wire [6:0] value_1, value_0;

    wire [2:0] curr_state;

    wire [3:0] ssd_in;

    reg [3:0] display_tmp_3;
    reg [3:0] display_tmp_2;
    reg [3:0] display_tmp_1;
    reg [3:0] display_tmp_0;

    assign valid_signal = key_down[last_change];

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst(rst)   //low-active reset
    );

    KeyboardDecoder U1(
	.key_down(key_down),
	.last_change(last_change),
	.key_valid(key_valid),
	.PS2_DATA(PS2_DATA),
	.PS2_CLK(PS2_CLK),
	.rst(rst),
	.clk(clk)
    );
    
    key_decoder U2(
    .result(decode_result), 
    .select_in(last_change),
    .clk(clk),
    .rst(rst)
    );

    fsm U3(
    .instruction(instruction),
    .valid(valid_signal),
    .curr_state(curr_state),
    .rst(rst),
    .decoded_input(decode_result)
);
    value_temp_reg U4(
    .digit_0(digit_0),
    .digit_1(digit_1),
    .digit_2(digit_2),
    .digit_3(digit_3),
    .state_sel(curr_state),
    .valid(valid_signal),
    .decoded_input(decode_result),
    .rst(rst)
);
    dec_adder U5(
    .cout(add_digit_2), 
    .num_0(digit_0), 
    .num_1(digit_1),
    .num_2(digit_2),
    .num_3(digit_3),
    .digit_0_out(add_digit_0),
    .digit_1_out(add_digit_1)
    );

    dec_subtractor U6(
    .num_0(digit_0), 
    .num_1(digit_1),
    .num_2(digit_2),
    .num_3(digit_3),
    .digit_0_out(subtract_digit_0),
    .digit_1_out(subtract_digit_1),
    .value_0(value_0),
    .value_1(value_1)
    );

    dec_multiplier U7(
    .num_0(digit_0), 
    .num_1(digit_1),
    .num_2(digit_2),
    .num_3(digit_3),
    .digit_0_out(multiply_digit_0),
    .digit_1_out(multiply_digit_1),
    .digit_2_out(multiply_digit_2),
    .digit_3_out(multiply_digit_3)
    );

    //display selector
    always @*
        if (curr_state == `FINAL_DISPLAY_STATE && instruction == 2'd0)
            begin
                display_tmp_3 = 4'd0;
                display_tmp_2 = add_digit_2;
                display_tmp_1 = add_digit_1;
                display_tmp_0 = add_digit_0;
            end
        else if (curr_state == `FINAL_DISPLAY_STATE && instruction == 2'd1 && (value_1 >= value_0))
            begin
                display_tmp_3 = 4'd0;
                display_tmp_2 = 4'd0;
                display_tmp_1 = subtract_digit_1;
                display_tmp_0 = subtract_digit_0;
            end
        else if (curr_state == `FINAL_DISPLAY_STATE && instruction == 2'd1 && (value_1 < value_0))
            begin
                display_tmp_3 = 4'd0;
                display_tmp_2 = 4'd10; //negative
                display_tmp_1 = subtract_digit_1;
                display_tmp_0 = subtract_digit_0;
            end
        else if (curr_state == `FINAL_DISPLAY_STATE && instruction == 2'd2)
            begin
                display_tmp_3 = multiply_digit_3;
                display_tmp_2 = multiply_digit_2;
                display_tmp_1 = multiply_digit_1;
                display_tmp_0 = multiply_digit_0;
            end
        else
            begin
                display_tmp_3 = digit_3;
                display_tmp_2 = digit_2;
                display_tmp_1 = digit_1;
                display_tmp_0 = digit_0;
            end
            
    scan_freq_ctl U9(
    .ssd_in(ssd_in),
    .ssd_ctl(ssd_ctl),
    .clk_ctl_en(clk_ssd_ctl),
    .in_3(display_tmp_3),
    .in_2(display_tmp_2),
    .in_1(display_tmp_1),
    .in_0(display_tmp_0)
);

    display U10(.D_ssd(D_ssd), .i(ssd_in));

endmodule
